@@ -15,11 +15,11 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.12';
+$VERSION = '1.15';
 
 sub ProcessMPImageList($$$);
 
-# Tags found in MPF APP2 segment in JPEG images
+# Tags found in APP2 MPF segment in JPEG images
 %Image::ExifTool::MPF::Main = (
     GROUPS => { 0 => 'MPF', 1 => 'MPF0', 2 => 'Image'},
     NOTES => q{
@@ -28,8 +28,8 @@ sub ProcessMPImageList($$$);
         from this segment is stored as a JPEG trailer.  The MPF tags are not
         writable, however the MPF segment may be deleted as a group (with "MPF:All")
         but then the JPEG trailer should also be deleted (with "Trailer:All").  See
-        L<http://www.cipa.jp/std/documents/e/DC-007_E.pdf> for the official
-        specification.
+        L<https://web.archive.org/web/20190713230858/http://www.cipa.jp/std/documents/e/DC-007_E.pdf>
+        for the official specification.
     },
     0xb000 => 'MPFVersion',
     0xb001 => 'NumberOfImages',
@@ -97,7 +97,7 @@ sub ProcessMPImageList($$$);
     NOTES => q{
         The first MPF "Large Thumbnail" image is extracted as PreviewImage, and the
         rest of the embedded MPF images are extracted as MPImage#.  The
-        ExtractEmbedded (-ee) option may be used to extract information from these
+        L<ExtractEmbedded|../ExifTool.html#ExtractEmbedded> (-ee) option may be used to extract information from these
         embedded images.
     },
     0.1 => {
@@ -105,9 +105,9 @@ sub ProcessMPImageList($$$);
         Format => 'int32u',
         Mask => 0xf8000000,
         PrintConv => { BITMASK => {
-            29 => 'Representative image',
-            30 => 'Dependent child image',
-            31 => 'Dependent parent image',
+            2 => 'Representative image',
+            3 => 'Dependent child image',
+            4 => 'Dependent parent image',
         }},
     },
     0.2 => {
@@ -131,6 +131,7 @@ sub ProcessMPImageList($$$);
             0x020002 => 'Multi-frame Disparity',
             0x020003 => 'Multi-angle',
             0x030000 => 'Baseline MP Primary Image',
+            0x040000 => 'Original Preservation Image', # (Exif 3.0)
         },
     },
     4 => {
@@ -163,12 +164,13 @@ sub ProcessMPImageList($$$);
         },
         Notes => q{
             the first MPF "Large Thumbnail" is extracted as PreviewImage, and the rest
-            of the embedded MPF images are extracted as MPImage#.  The ExtractEmbedded
+            of the embedded MPF images are extracted as MPImage#.  The L<ExtractEmbedded|../ExifTool.html#ExtractEmbedded>
             option may be used to extract information from these embedded images.
         },
         # extract all MPF images (not just one)
         RawConv => q{
             require Image::ExifTool::MPF;
+            @grps = $self->GetGroup($$val{0});  # set groups from input tag
             Image::ExifTool::MPF::ExtractMPImages($self);
         },
     },
@@ -211,12 +213,7 @@ sub ExtractMPImages($)
                     Groups => { 0 => 'Composite', 1 => 'Composite', 2 => 'Preview'},
                 });
             }
-            my $key = $et->FoundTag($tag, $val);
-            # set groups for PreviewImage
-            if ($tag eq 'PreviewImage') {
-                $et->SetGroup($key, 'Composite', 0);
-                $et->SetGroup($key, 'Composite');
-            }
+            my $key = $et->FoundTag($tag, $val, $et->GetGroup("MPImageStart$xtra"));
             # extract information from MP images if ExtractEmbedded option used
             if ($ee) {
                 my $oldBase = $$et{BASE};
@@ -270,7 +267,7 @@ Format (MPF) information from JPEG images.
 
 =head1 AUTHOR
 
-Copyright 2003-2016, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2025, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
